@@ -94,7 +94,7 @@ int compress = 0;
 module_param(compress, int, S_IRUGO);
 #endif
 
-int init_module (void)
+static int __init lime_init_module (void)
 {
     if(!path) {
         DBG("No path parameter specified");
@@ -280,7 +280,11 @@ static void write_range(struct resource * res) {
 #endif
         p = pfn_to_page((i) >> PAGE_SHIFT);
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,18)
+        is = min((resource_size_t) PAGE_SIZE, (resource_size_t) (res->end - i + 1));
+#else
         is = min((size_t) PAGE_SIZE, (size_t) (res->end - i + 1));
+#endif
 
         if (is < PAGE_SIZE) {
             // We can't map partial pages and
@@ -288,19 +292,30 @@ static void write_range(struct resource * res) {
             DBG("Padding partial page: vaddr %p size: %lu", (void *) i, (unsigned long) is);
             write_padding(is);
         } else {
+#ifdef LIME_USE_KMAP_ATOMIC
+            v = kmap_atomic(p);
+#else
             v = kmap(p);
+#endif
             /*
              * If we need to compute the digest or compress the output
              * take a snapshot of the page. Otherwise save some cycles.
              */
+#ifdef LIME_USE_KMAP_ATOMIC
+            preempt_enable();
+#endif
             if (no_overlap) {
                 copy_page(vpage, v);
                 s = write_vaddr(vpage, is);
             } else {
                 s = write_vaddr(v, is);
             }
+#ifdef LIME_USE_KMAP_ATOMIC
+            preempt_disable();
+            kunmap_atomic(v);
+#else
             kunmap(p);
-
+#endif
             if (s < 0) {
                 DBG("Failed to write page: vaddr %p. Skipping Range...", v);
                 break;
@@ -379,8 +394,11 @@ static void cleanup(void) {
     return (method == LIME_METHOD_TCP) ? cleanup_tcp() : cleanup_disk();
 }
 
-void cleanup_module(void) {
+static void __exit lime_cleanup_module(void) {
 
 }
 
+module_init(lime_init_module);
+module_exit(lime_cleanup_module);
+
 MODULE_LICENSE("GPL");
